%{

//  ASL COMPILER
//
//  ASL (Abstract Schema Language) to C++ compiler
//
//
//
//  FILE NAME   : aslclex.l
//  MODULE TYPE : Lex input specification
//  STATUS      : In development
//  PURPOSE     : Lexical analisys
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1995-1998
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.00  (Apr 20, 1992 - Longguang Wu), 
//            Start of development.
//
//            0.50  (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 alpha.
//
//            1.00  (Dec 17, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//	      1.05  (Oct 09, 1997 - Oscar Peguero),
//	      Changes for the NSLC 3.0 specifications.
//
//	      1.10  (Feb 16, 1998 - Sebastian Gutierrez),
//	      Sintax extended according to the NSLC 3.0 specifications.
// 

#include <string.h>

#include "aslc.h"
#include "y.tab.h"           // Generated by yacc.


// Global variables.
extern AslCompiler compiler;
extern int flagObject;
extern int flagAssign;
extern int flagSchema;
extern char myArgument[STR_LEN_LINE];
extern char myStack[STR_LEN_LINE];
extern char myAssign[STR_LEN_LINE];

%}

/* Lex definitions. */
	       
	       /* Identifiers: variables, keywords, etc. */
NAME           [a-zA-Z\_][a-zA-Z0-9\_]* 

	       /* Integer constants. */
NUMBER         [0-9]

	       /* String constants. */
STRING         \"[^\"]*\"

	       /* Special characters. */
SPECIAL        [\&\=\{\}\|\[\]\\\;\'\:\"\?\.\/]

MYSPECIAL	[\(\)\,\*\-\+\%\^\!\>\<]


%x ST_SCHEMA

%%
"#".*\n                        {
					compiler.parser.processIncludeLines(yytext);

					sprintf(compiler.buf,"#line %d \"%s\"\n", compiler.parser.getNumOfLines(), compiler.getInputFileName());
					compiler.putInOutFile(compiler.buf);
				}

\n.*                            { 
					compiler.putInOutFile("\n");
					compiler.parser.saveLine(yytext+1);
					compiler.parser.incNumOfLines();
					compiler.parser.setLinePos(0);
					yyless(1);
				}

nslModule			{
					compiler.putInOutFile("class");
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SCHEMA);

					return SCHEMA;
				}

nslClass			{
					compiler.putInOutFile("class");
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SCHEMA);
					
					return SCHEMA;
				}


class				{
					compiler.putInOutFile("class");
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SCHEMA);
				
					return SCHEMA;
				}
				
external			{
					compiler.putInOutFile("");
					compiler.parser.addLinePos(yyleng);
				}

internal			{
					compiler.putInOutFile("");
					compiler.parser.addLinePos(yyleng);
				}
				
static				{ }				

{NAME}                          {
					compiler.putInOutFile(yytext);
				}

{STRING}                        {
					/* String constant */
					compiler.putInOutFile(yytext);
				}

"-"?{NUMBER}+([ULul]*)?         {
					/* Integer constant */

					compiler.putInOutFile(yytext);
				}

"-"?"0x"({NUMBER}|[a-fA-F])+([ULul]*)? {
					/* Integer constant (hex) */

					compiler.putInOutFile(yytext);
				}

"-"?{NUMBER}*"."{NUMBER}+((e|E)"-"?{NUMBER}+)?([fFlL]*)?        {
					/* Float constant */

					compiler.putInOutFile(yytext);
				}

"-"?{NUMBER}+((e|E)"-"?{NUMBER}+)([fFlL]*)?     {
					/* Float constant */

					compiler.putInOutFile(yytext);
				}

"'"."'"                         {
					/* Character constant */

					compiler.putInOutFile(yytext);
				}

[ \t\r]                         {
					/* Ignore blanks. */
					compiler.putInOutFile(yytext);
				}

{SPECIAL}                       {
					compiler.buf[0]=yytext[0];
					compiler.buf[1]='\0';

					compiler.putInOutFile(compiler.buf);
				} 

.                               {
					if (isprint(yytext[0]))
						sprintf(compiler.buf,"Illegal character '%c' (%x)\n",yytext[0], yytext[0]);
					else
						sprintf(compiler.buf,"Illegal character (\\%x)\n",yytext[0]);

					compiler.parser.addLinePos(yyleng);
					compiler.sintaxError(compiler.buf);
				}

<ST_SCHEMA>"#".*\n		{
					compiler.parser.processIncludeLines(yytext+1);

					sprintf(compiler.buf,"#line &d \"%s\"", compiler.parser.getNumOfLines(), compiler.getInputFileName());
					compiler.putInOutFile(compiler.buf);
				}

<ST_SCHEMA>\n.*                 {
					compiler.putInOutFile("\n");
					compiler.parser.saveLine(yytext+1);
					compiler.parser.incNumOfLines();
					compiler.parser.setLinePos(0);
					yyless(1);
				}

<ST_SCHEMA>class		{
					compiler.parser.addLinePos(yyleng);

					return SCHEMA;
				}
				
<ST_SCHEMA>constructor		{ }

<ST_SCHEMA>delegate             {
					compiler.putInOutFile("DELEGATE");
					compiler.parser.addLinePos(yyleng);

					return DELEGATE;
				}

<ST_SCHEMA>destructor           { }

<ST_SCHEMA>external             {
					compiler.putInOutFile("public :");
					compiler.parser.addLinePos(yyleng);

					return EXTERNAL;
				}

<ST_SCHEMA>extends              {
					compiler.putInOutFile(": public");
					compiler.parser.addLinePos(yyleng);

					return INHERIT;
				}

<ST_SCHEMA>input                {
					compiler.putInOutFile("INPUT");
					compiler.parser.addLinePos(yyleng);

					return INPUT;
				}

<ST_SCHEMA>internal             {
					compiler.putInOutFile("private :");
					compiler.parser.addLinePos(yyleng);

					return INTERNAL;
				}

<ST_SCHEMA>nslModule		{
					compiler.parser.addLinePos(yyleng);

					return SCHEMA;
				}

<ST_SCHEMA>output               {
					compiler.putInOutFile("OUTPUT");
					compiler.parser.addLinePos(yyleng);

					return OUTPUT;
				}

<ST_SCHEMA>private:             {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return INTERNAL;
				}

<ST_SCHEMA>public:             {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return EXTERNAL;
				}
				
<ST_SCHEMA>sin                  {
					compiler.putInOutFile("aslio");
					compiler.parser.addLinePos(yyleng);
				}

<ST_SCHEMA>sout                 {
					compiler.putInOutFile("aslio");
					compiler.parser.addLinePos(yyleng);
				}

<ST_SCHEMA>stop                 {
					compiler.putInOutFile("STOP");
					compiler.parser.addLinePos(yyleng);

					return STOP;
				}

<ST_SCHEMA>this                 {
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else
					  {
						strcpy(myStack,"");
						compiler.putInOutFile(yytext);
					  }
					compiler.parser.addLinePos(yyleng);
				

					return THIS;
				}

<ST_SCHEMA>">=>"                {
					compiler.putInOutFile("CONNECT_LR");
					compiler.parser.addLinePos(yyleng);

					return OP_CONNECT_LR;
				}

<ST_SCHEMA>"<=<"                {
					compiler.putInOutFile("CONNECT_RL");
					compiler.parser.addLinePos(yyleng);

					return OP_CONNECT_RL;
				}

<ST_SCHEMA>">=<"                {
					compiler.putInOutFile("DISCONNECT");
					compiler.parser.addLinePos(yyleng);

					return OP_DISCONNECT;
				}

<ST_SCHEMA>"==="                {
					compiler.putInOutFile("RELABLE");
					compiler.parser.addLinePos(yyleng);

					return OP_RELABEL;
				}

<ST_SCHEMA>"=|="                {
					compiler.putInOutFile("DELABEL");
					compiler.parser.addLinePos(yyleng);

					return OP_DELABEL;
				}


<ST_SCHEMA>auto                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return AUTO;
				}

<ST_SCHEMA>break                {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return BREAK;
				}

<ST_SCHEMA>case                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return CASE;
				}

<ST_SCHEMA>char                 {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return CHAR;
				}

<ST_SCHEMA>const                {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return CONST;
				}

<ST_SCHEMA>continue             {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return CONTINUE;
				}

<ST_SCHEMA>default              {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return DEFAULT;
				}

<ST_SCHEMA>delete               {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return DELETE;
				}

<ST_SCHEMA>do                   {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return DO;
				}

<ST_SCHEMA>double               {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return DOUBLE;
				}

<ST_SCHEMA>else                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return ELSE;
				}

<ST_SCHEMA>extern               {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return EXTERN;
				}

<ST_SCHEMA>float                {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return FLOAT;
				}

<ST_SCHEMA>for                  {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return FOR;
				}

<ST_SCHEMA>if                   {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return IF;
				}
		    
<ST_SCHEMA>int                  {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return INT;
				}

<ST_SCHEMA>long                 {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return LONG;
				}
			
<ST_SCHEMA>new                  {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NEW;
				}

<ST_SCHEMA>NslDinDouble       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE;
				}

<ST_SCHEMA>NslDinDouble0       	{
					if (!flagSchema) compiler.putInOutFile("NslDinDouble");
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE;
				}

<ST_SCHEMA>NslDinDouble1       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE1;
				}

<ST_SCHEMA>NslDinDouble2       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE2;
				}

<ST_SCHEMA>NslDinDouble3       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE3;
				}

<ST_SCHEMA>NslDinDouble4       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE4;
				}

<ST_SCHEMA>NslDinFloat     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT;
				}

<ST_SCHEMA>NslDinFloat0       	{
					if (!flagSchema) compiler.putInOutFile("NslDinFloat");
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT;
				}

<ST_SCHEMA>NslDinFloat1       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT1;
				}

<ST_SCHEMA>NslDinFloat2       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT2;
				}

<ST_SCHEMA>NslDinFloat3    	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT3;
				}

<ST_SCHEMA>NslDinFloat4     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT4;
				}

<ST_SCHEMA>NslDinInt     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDININT;
				}

<ST_SCHEMA>NslDinInt0    	{
					if (!flagSchema) compiler.putInOutFile("NslDinInt");
					compiler.parser.addLinePos(yyleng);

					return NSLDININT;
				}

<ST_SCHEMA>NslDinInt1    	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDININT1;
				}

<ST_SCHEMA>NslDinInt2     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDININT2;
				}

<ST_SCHEMA>NslDinInt3     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDININT3;
				}

<ST_SCHEMA>NslDinInt4     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDININT4;
				}

<ST_SCHEMA>NslDoutDouble       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE;
				}

<ST_SCHEMA>NslDoutDouble0      	{
					if (!flagSchema) compiler.putInOutFile("NslDoutDouble");
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE;
				}

<ST_SCHEMA>NslDoutDouble1      	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE1;
				}

<ST_SCHEMA>NslDoutDouble2      	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE2;
				}

<ST_SCHEMA>NslDoutDouble3      	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE3;
				}

<ST_SCHEMA>NslDoutDouble4      	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE4;
				}

<ST_SCHEMA>NslDoutFloat         {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT;
				}

<ST_SCHEMA>NslDoutFloat0       	{
					if (!flagSchema) compiler.putInOutFile("NslDoutFloat");
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT;
				}

<ST_SCHEMA>NslDoutFloat1       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT1;
				}

<ST_SCHEMA>NslDoutFloat2       	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT2;
				}

<ST_SCHEMA>NslDoutFloat3    	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT3;
				}

<ST_SCHEMA>NslDoutFloat4     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT4;
				}

<ST_SCHEMA>NslDoutInt     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT;
				}

<ST_SCHEMA>NslDoutInt0    	{
					if (!flagSchema) compiler.putInOutFile("NslDoutInt");
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT;
				}

<ST_SCHEMA>NslDoutInt1    	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT1;
				}

<ST_SCHEMA>NslDoutInt2     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT2;
				}

<ST_SCHEMA>NslDoutInt3     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT3;
				}

<ST_SCHEMA>NslDoutInt4     	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT4;
				}

<ST_SCHEMA>NslDouble           	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE;
				}

<ST_SCHEMA>NslDouble0          	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE0;
				}

<ST_SCHEMA>NslDouble1          	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE1;
				}

<ST_SCHEMA>NslDouble2          	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE2;
				}

<ST_SCHEMA>NslDouble3          	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE3;
				}

<ST_SCHEMA>NslDouble4          	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE4;
				}

<ST_SCHEMA>NslFloat            	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT;
				}

<ST_SCHEMA>NslFloat0           	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT0;
				}

<ST_SCHEMA>NslFloat1           	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT1;
				}

<ST_SCHEMA>NslFloat2           	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT2;
				}

<ST_SCHEMA>NslFloat3           	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT3;
				}

<ST_SCHEMA>NslFloat4           	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT4;
				}

<ST_SCHEMA>NslInt           	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLINT;
				}

<ST_SCHEMA>NslInt0          	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLINT0;
				}

<ST_SCHEMA>NslInt1         	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLINT1;
				}

<ST_SCHEMA>NslInt2          	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLINT2;
				}

<ST_SCHEMA>NslInt3         	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLINT3;
				}

<ST_SCHEMA>NslInt4         	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLINT4;
				}

<ST_SCHEMA>NslString           	{
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NSLSTRING;
				}


<ST_SCHEMA>register             {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return REGISTER;
				}

<ST_SCHEMA>return               {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return RETURN;
				}

<ST_SCHEMA>short                {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return SHORT;
				}

<ST_SCHEMA>signed               {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return SIGNED;
				}


<ST_SCHEMA>sizeof               {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return SIZEOF;
				}

<ST_SCHEMA>static               {
					compiler.putInOutFile("");	/* antes yytext y return STATIC */
				}

<ST_SCHEMA>switch               {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return SWITCH;
				}

<ST_SCHEMA>unsigned             {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return UNSIGNED;
				}

<ST_SCHEMA>void                 {
					if (!flagSchema) compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return VOID;
				}

<ST_SCHEMA>volatile             {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return VOLATILE;
				}

<ST_SCHEMA>while                {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return WHILE;
				}

<ST_SCHEMA>"*="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return MULT_EQUAL;
				}

<ST_SCHEMA>"/="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return DIV_EQUAL;
				}

<ST_SCHEMA>"%="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return MOD_EQUAL;
				}

<ST_SCHEMA>"+="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return PLUS_EQUAL;
				}

<ST_SCHEMA>"-="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return MINUS_EQUAL;
				}

<ST_SCHEMA>">>="                {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return SHIFT_RIGHT_EQUAL;
				}

<ST_SCHEMA>"<<="                {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return SHIFT_LEFT_EQUAL;
				}

<ST_SCHEMA>"&="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return AND_EQUAL;
				}

<ST_SCHEMA>"^="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return XOR_EQUAL;
				}

<ST_SCHEMA>"|="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return OR_EQUAL;
				}

<ST_SCHEMA>"||"                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return OR_LOGIC;
				}

<ST_SCHEMA>"&&"                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return AND_LOGIC;
				}

<ST_SCHEMA>"=="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return EQUAL;
				}

<ST_SCHEMA>"!="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return NOT_EQUAL;
				}

<ST_SCHEMA>"<<"                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return SHIFT_LEFT;
				}

<ST_SCHEMA>">>"                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return SHIFT_RIGHT;
				}

<ST_SCHEMA>"<="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return LOWER_EQUAL;
				}

<ST_SCHEMA>">="                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return GREATER_EQUAL;
				}

<ST_SCHEMA>"++"                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return PLUS_PLUS;
				}

<ST_SCHEMA>"--"                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return MINUS_MINUS;
				}

<ST_SCHEMA>"."	                {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return INDIRECTION_1;
				}

ST_SCHEMA>"->"	                {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return INDIRECTION_1;
				}


<ST_SCHEMA>".*"                 {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return INDIRECTION_2;
				}

<ST_SCHEMA>"->*"                {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					return INDIRECTION_3;
				}

<ST_SCHEMA>"\"*\""                 {
					compiler.putInOutFile("");
					compiler.parser.addLinePos(yyleng);

					return SELF;
				}

<ST_SCHEMA>{NAME}               {
					int type;
					if (!flagObject)
					  {
						if (!flagSchema) compiler.putInOutFile(yytext);
				     	   }
					if ((flagObject) && (flagAssign))
					  {
						/*compiler.putInOutFile(yytext);*/
						if (flagSchema) 
						  {
							compiler.putInOutFile(yytext);
							flagSchema = FALSE;
							flagAssign = FALSE;
						  }
					  }
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name, yytext);

					type = compiler.parser.whichTypeIs(yytext);

					switch (type)
					{
						case SCHEMA_SYMBOL: 	{
									/*printf("SCHEMA %s\n",yylval.name);*/
									  return SCHEMA_NAME;
									}
						case STRUCT_SYMBOL:
									{
									/*printf("STRUCT %s\n",yylval.name);*/
									  return STRUCT_NAME;}
						case FUNCTION_SYMBOL:
									{
									/*printf("FUNCTION  %s\n",yylval.name);*/
									  return FUNCTION_NAME;}
						case IDENT_SYMBOL:
									{
									/*printf("ID_DECLARED %s\n",yylval.name);*/
									  return IDENTIFIER_DECLARED;}
						default:
									{
									/*printf("IDENTIFIER %s\n",yylval.name);*/
									  return IDENTIFIER;
									}
								
					}
				}

<ST_SCHEMA>{STRING}             {
					
					if (flagObject) 
					  {
					 	 strcpy(myStack,yytext) ;
					  }
					else 
					  {
						strcpy(myStack,"");
						if (!flagSchema) compiler.putInOutFile(yytext);
					  }
					compiler.parser.addLinePos(yyleng);

					return CONST_STRING;
				}

<ST_SCHEMA>{NUMBER}+([ULul]*)?  {
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else 
					  {
						strcpy(myStack,"");
						if (!flagSchema) compiler.putInOutFile(yytext);
					  }
					compiler.parser.addLinePos(yyleng);
					
					return CONST_INT;
				}

<ST_SCHEMA>"0x"({NUMBER}|[a-fA-F])+([ULul]*)?   {
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else
					  {
						strcpy(myStack,"");
						if (!flagSchema) compiler.putInOutFile(yytext);
					  }
					compiler.parser.addLinePos(yyleng);

					return CONST_INT;
				}

<ST_SCHEMA>{NUMBER}*"."{NUMBER}+((e|E)"-"?{NUMBER}+)?([fFlL]*)? {
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else
					  {
						strcpy(myStack,"");
						if (!flagSchema) compiler.putInOutFile(yytext);
					  }
					compiler.parser.addLinePos(yyleng);

					return CONST_FLOAT;
				}

<ST_SCHEMA>{NUMBER}+((e|E)"-"?{NUMBER}+)([fFlL]*)? {
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else
					  {
						strcpy(myStack,"");
						if (!flagSchema) compiler.putInOutFile(yytext);
					  }
					compiler.parser.addLinePos(yyleng);

					return CONST_FLOAT;
				}

<ST_SCHEMA>"'"."'"              {
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else
					  {
						strcpy(myStack,"");
						if (!flagSchema) compiler.putInOutFile(yytext);
					  }
					compiler.parser.addLinePos(yyleng);
					
					return CONST_CHAR;
				}

<ST_SCHEMA>[ \t\r]              {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					/* Ignore blanks. */
				}


<ST_SCHEMA>{SPECIAL}            {
					compiler.buf[0]=yytext[0];
					compiler.buf[1]='\0';
					if ((strcmp(yytext,"=") == 0) && (!flagAssign) && (flagObject)) 
					  {
						/*printf("=");*/
						/*strcpy(myAssign,"");*/
					  	flagAssign = TRUE;
					  }
					else
					  {
						if (!flagSchema) compiler.putInOutFile(compiler.buf);
					  }	
					if ((strcmp(yytext,";") == 0) && (flagAssign == TRUE)) 
					  {
						/*printf(";");*/
						strcat(myAssign,";\n");
					  }
					compiler.parser.addLinePos(yyleng);
					
					return yytext[0];
				}

<ST_SCHEMA>{MYSPECIAL}        	{
					if (!flagObject)
					  {
						compiler.buf[0]=yytext[0];
						compiler.buf[1]='\0';	
						if ((strcmp(yytext,"(") == 0) && (flagAssign == 2)) 
						  {
							/*printf("(\n");*/
							flagAssign = FALSE;
						  }
						else
						  {
							if (!flagSchema) compiler.putInOutFile(compiler.buf);
						  }	
					  }
					compiler.parser.addLinePos(yyleng);

					return yytext[0];
				}

<ST_SCHEMA>.                    {
					if (isprint(yytext[0]))
						sprintf(compiler.buf,"Illegal character '%c' (%x)\n",yytext[0],yytext[0]);
					else
						sprintf(compiler.buf,"Illegal character (\\%x)\n",yytext[0]);

					compiler.sintaxError(compiler.buf);
					compiler.parser.addLinePos(yyleng);
				}
%%

int yywrap(void)
{
	return TRUE;
}

void AslParser::setLexState(int p)
{
	BEGIN(p);
}
