/* SCCS  @(#)nsl_input_3d.C	1.1---95/08/13--20:55:05 */
//
//	nsl_input_3d.c
//

# include "nsl_all_kernel_include.h"
# include "nsl_all_input_include.h"

nsl_input_3d::nsl_input_3d(const char* str,nsl_public* parent,
	const int imx,const int jmx,const int izx,const int jzx,int sfg,int fg,int rfg) : 
	nsl_input_matrix(str,parent,imx,jmx,sfg,fg,0), // rfg
	zz("zz",this),	// matrix zero elements
	dz("dz",this),
	xzview("xzview",this,izx,jzx),
	transx("transx",this,1), // translation to origin
	transy("transy",this,1),
	angle("angle",this,1)	// angle of rotation in radians: degrees/180=radians/PI
{
	set_type("input_3d");

	zz = 0;
	dz = 1.0;
}	
nsl_input_3d::~nsl_input_3d()
{
}

// input 3d assignment (input_3d = 3d is NOT derived !!)

NslFloat2& nsl_input_3d::operator=(const nsl_input_3d& a)
{
	if (a.get_imax() != imax || a.get_jmax() != jmax)
	{
		cmd_error("Error in nsl_input_3d::operator=");
 		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error("Inconsistent 3d sizes");
		return *this;
	}

	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		(*v[i])[j] = a[i][j];

	return *this;
}
NslFloat2& nsl_input_3d::operator=(const NslFloat2& a)
{
	if (a.get_imax() != imax || a.get_jmax() != jmax)
	{
		cmd_error("Error in nsl_input_3d::operator=");
		a.print_status(cerr);
		this->print_status(cerr);
		cmd_error("Inconsistent 3d sizes");
		return *this;
	}
	
	for (int i = 0; i < imax; i++)
	   for (int j = 0; j < jmax; j++)
		(*v[i])[j] = a[i][j];

	return *this;
}
NslFloat2& nsl_input_3d::operator=(const NslFloat0& a)
{
	int imx = get_imax();  
	int jmx = get_jmax();

	num_type val = a.get_value();

	for (int i = 0; i < imx; i++)
	   for (int j = 0; j < jmx; j++)
		(*v[i])[j] = val;

	return *this;
}
NslFloat2& nsl_input_3d::operator=(const num_type a)
{
	int imx = get_imax();
	int jmx = get_jmax();

	for (int i = 0; i < imx; i++)
	   for (int j = 0; j < jmx; j++)
		(*v[i])[j] = a;

	return *this;
}
NslFloat2& nsl_input_3d::operator=(const int a)
{
	int imx = get_imax();
	int jmx = get_jmax();

	for (int i = 0; i < imx; i++)
	   for (int j = 0; j < jmx; j++)
		(*v[i])[j] = (num_type) a;

	return *this;
}

int nsl_input_3d::run()
{
	if (input_layer != NULL) {
		xzview = 0; // reset camera view
		input_layer->run();
	}
	else
		cmd_error("NULL input stim");

	return 1;
}

// print status

int nsl_input_3d::print_status(ostream& out) const
{
	return print_input_3d_status(out);
}
int nsl_input_3d::print_input_3d_status(ostream& out) const
{
 	nsl_input_matrix::print_status(out);

	if (input_layer != NULL)
		input_layer->print_status(out);
	else
		cmd_error("NULL input stim");

	return 1;
}
int nsl_input_3d::print(ostream& out) const
{
	nsl_input_matrix::print(out);

  	out << "set input_layer xz " << xz.get_value() << "\n";
	out << "set input_layer yz " << yz.get_value() << "\n";
	out << "set input_layer dx " << dx.get_value() << "\n";
	out << "set input_layer dy " << dy.get_value() << "\n";
 
	out.flush();

	if (input_layer != NULL)
		input_layer->print(out);
	else
		cmd_error("NULL input stim");

	return 1;
}
int nsl_input_3d::print(ostream& out,int i) const
{
	nsl_input_matrix::print(out);

  	out << "set input_layer xz " << xz.get_value() << "\n";
	out << "set input_layer yz " << yz.get_value() << "\n";
	out << "set input_layer dx " << dx.get_value() << "\n";
	out << "set input_layer dy " << dy.get_value() << "\n";
 
	out.flush();

	if (input_layer != NULL)
		input_layer->print(out,i);
	else
		cmd_error("NULL input stim");

	return 1;
}
