/* SCCS  @(#)nsl_mat_vec_prod_library.h	1.2---95/08/14--16:08:57 */
/****************************************************************/
/*                                                           	*/
/*                  nsl_mat_vec_prod_library.h                  */
/*                                                            	*/
/****************************************************************/

template<class NslNumType, int NslTypeId>
nsl_num_2<NslNumType,NslTypeId> NSLprod2(const nsl_num_1<NslNumType,NslTypeId>& v1,const NslDouble1& v2)
{
	int size1 = v1.get_imax();
	int size2 = v2.get_imax();

	nsl_num_2<NslNumType,NslTypeId> c(size2,size1);
	
	for (int i = 0; i < size1; i++)
		for (int j = 0; j < size2; j++)
			c[j][i] = v1[i]*v2[j];

	return c;
}
template<class NslNumType, int NslTypeId>
nsl_num_1<NslNumType,NslTypeId> NSLprod2(const nsl_num_2<NslNumType,NslTypeId>& w,const NslDouble1& x)
{
	int size = w.get_imax();
	int vsize = w.get_jmax();

	nsl_num_1<NslNumType,NslTypeId> c(vsize);

	if (size != x.get_imax())
	{
        cmd_error("*** NSLprod ***");
        cmd_error("Exit with the following status");
        w.print_status(cerr);
        x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
		
	for (int i = 0; i < vsize; i++)
	{
	    c[i] = 0;
	    for (int j = 0; j < size; j++)
		c[i] = c[i] + w[j][i]*x[j];
	}

	return c;
}

// product

template<class NslNumType, int NslTypeId>
nsl_num_0<NslNumType,NslTypeId> NSLprod(const nsl_num_1<NslNumType,NslTypeId>& w,const nsl_num_1<NslNumType,NslTypeId>& x)
{
	int size = w.get_imax();

	nsl_num_0<NslNumType,NslTypeId> c;

	if (size != x.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
	
	for (int i = 0; i < size; i++)
		c = c + w[i]*x[i];

	return c;
}

template<class NslNumType, int NslTypeId>
nsl_num_1<NslNumType,NslTypeId> NSLprod(const nsl_num_1<NslNumType,NslTypeId>& w,const nsl_num_2<NslNumType,NslTypeId>& x)
{
	int size = x.get_imax();
	int vsize = x.get_jmax();

	nsl_num_1<NslNumType,NslTypeId> c(vsize);

	if (size != w.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
	
	for (int j = 0; j < vsize; j++)
	{
	    c[j] = 0;
	    for (int i = 0; i < size; i++)
		c[j] = c[j] + w[i]*x[i][j];
	}

	return c;
}

template<class NslNumType, int NslTypeId>
nsl_num_1<NslNumType,NslTypeId> NSLprod(const nsl_num_2<NslNumType,NslTypeId>& w,const NslFloat1& x)
{
	int size = w.get_jmax();
	int vsize = w.get_imax();

	nsl_num_1<NslNumType,NslTypeId> c(vsize);

	if (size != x.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
		
	for (int i = 0; i < vsize; i++)
	{
	    c[i] = 0;
	    for (int j = 0; j < size; j++)
		c[i] = c[i] + w[i][j]*x[j];
	}

	return c;
}
template<class NslNumType, int NslTypeId>
nsl_num_1<NslNumType,NslTypeId> NSLprod(const nsl_num_2<NslNumType,NslTypeId>& w,const NslDouble1& x)
{
	int size = w.get_jmax();
	int vsize = w.get_imax();

	nsl_num_1<NslNumType,NslTypeId> c(vsize);

	if (size != x.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
		
	for (int i = 0; i < vsize; i++)
	{
	    c[i] = 0;
	    for (int j = 0; j < size; j++)
		c[i] = c[i] + w[i][j]*x[j];
	}

	return c;
}
template<class NslNumType, int NslTypeId>
nsl_num_1<NslNumType,NslTypeId> NSLprod(const nsl_num_2<NslNumType,NslTypeId>& w,const NslInt1& x)
{
	int size = w.get_jmax();
	int vsize = w.get_imax();

	nsl_num_1<NslNumType,NslTypeId> c(vsize);

	if (size != x.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
		
	for (int i = 0; i < vsize; i++)
	{
	    c[i] = 0;
	    for (int j = 0; j < size; j++)
		c[i] = c[i] + w[i][j]*x[j];
	}

	return c;
}
template<class NslNumType, int NslTypeId>
nsl_num_2<NslNumType,NslTypeId> NSLprod(const nsl_num_2<NslNumType,NslTypeId>& w,const nsl_num_2<NslNumType,NslTypeId>& x)
{
	int isize = w.get_imax();
	int jsize = x.get_jmax();

	int size = w.get_jmax();

	nsl_num_2<NslNumType,NslTypeId> c(isize,jsize);

	if (size != x.get_imax())
	{
                cmd_error("*** NSLprod ***");
                cmd_error("Exit with the following status");
                w.print_status(cerr);
                x.print_status(cerr);
		cmd_error("Bad dimensions in NSLprod");
		return c;
	}
		
	for (int i = 0; i < isize; i++)
	    for (int j = 0; j < jsize; j++)
	    {
	    	c[i][j] = 0;
	    	for (int k = 0; k < size; k++)
		   c[i][j] = c[i][j] + w[i][k]*x[k][j];
	    }

	return c;
}
